unit PrintObj;

interface

uses Windows, Classes, Graphics, Printers, SysUtils, ComCtrls;

type
  TPrintObj = class
  protected
    FPrinter: TPrinter;
    FTitle: string;
    FPageFormat: string;
    FListView: TListView;
    FCurY: Integer;
    FMaxX: Integer;
    FMaxY: Integer;
    FHeight: Integer;
    FStarts: array of Integer;
    FWidths: array of Integer;

    procedure NewLine;
    procedure PrintHeader;
    procedure PrintLine(AItem: TListItem);
    procedure PrintRect(const S: string; Column: Integer);
  public
    constructor Create(APrinter: TPrinter; AListView: TListView);
    destructor  Destroy; override;

    procedure Print(const ATitle: string = ''; const APageFormat: string = '');
  end;

implementation

{ TPrintObj }

constructor TPrintObj.Create(APrinter: TPrinter; AListView: TListView);
begin
  inherited Create;
  FPrinter := APrinter;
  FListView := AListView;
end;

destructor TPrintObj.Destroy;
begin
  // FPrinter & FListView are not owned by TPrintObj
  inherited Destroy;
end;

procedure TPrintObj.NewLine;
begin
  Inc(FCurY, FHeight);
  if FCurY > FMaxY - 2 * FHeight then
  begin
    FCurY := 0;
    FPrinter.NewPage;
  end;
end;

procedure TPrintObj.Print(const ATitle, APageFormat: string);
var
  I, FConvX1, FConvX2: Integer;
begin
  FPrinter.BeginDoc;
  try
    FPrinter.Canvas.Font := FListView.Font;
    FTitle := ATitle;
    FPageFormat := APageFormat;
    FCurY := 0;
    FMaxX := FPrinter.PageWidth;
    FMaxY := FPrinter.PageHeight;
    FConvX1 := FPrinter.Canvas.TextWidth('M');
    FConvX2 := FListView.Canvas.TextWidth('M');
    SetLength(FStarts, FListView.Columns.Count);
    SetLength(FWidths, FListView.Columns.Count);
    for I := Low(FStarts) to High(FStarts) do
    begin
      if I = Low(FStarts) then
        FStarts[I] := FConvX1
      else
        FStarts[I] := FStarts[I - 1] + FWidths[I - 1];
      FWidths[I] := FListView.Columns[I].Width * FConvX1 div FConvX2;
    end;
    for I := 0 to FListView.Items.Count - 1 do
    begin
      if FCurY = 0 then
        PrintHeader;
      PrintLine(FListView.Items[I]);
    end;
  finally
    FPrinter.EndDoc;
  end;
end;

procedure TPrintObj.PrintHeader;
var
  I: Integer;
  R: TRect;
begin
  with FPrinter.Canvas do
  begin
    if (FTitle <> '') or (FPageFormat <> '') then
    begin
      Font.Size := Font.Size + 2;
      try
        FHeight := TextHeight(FTitle + FPageFormat + '1234567890');
        R := Rect(FStarts[0], FCurY, FMaxX, FCurY + FHeight);
        if FTitle <> '' then
          DrawText(Handle, PChar(FTitle),
            -1, R, DT_LEFT or DT_TOP or DT_SINGLELINE);
        if FPageFormat <> '' then
          DrawText(Handle, PChar(Format(FPageFormat, [FPrinter.PageNumber])),
            -1, R, DT_RIGHT or DT_TOP or DT_SINGLELINE);
        NewLine;
      finally
        Font.Size := Font.Size - 2;
      end;
      NewLine;
    end;
    FHeight := 0;
    Font.Style := [fsBold];
    try
      for I := 0 to FListView.Columns.Count - 1 do
        PrintRect(' ' + FListView.Columns[I].Caption + ' ', I);
      NewLine;
    finally
      Font.Style := [];
    end;
  end;
end;

procedure TPrintObj.PrintLine(AItem: TListItem);
var
  I: Integer;
  AStr: string;
begin
  FHeight := 0;
  for I := 0 to FListView.Columns.Count - 1 do
  begin
    if I = 0 then
      AStr := AItem.Caption
    else
      AStr := AItem.SubItems[I - 1];
    PrintRect(' ' + AStr + ' ', I);
  end;
  NewLine;
end;

procedure TPrintObj.PrintRect(const S: string; Column: Integer);
const
  AlignFlags: array [TAlignment] of Integer =
    (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  CurHeight: Integer;
  TwoEms: Integer;
  R: TRect;
begin
  with FPrinter.Canvas do
  begin
    CurHeight := TextHeight(S);
    TwoEms := TextWidth('M');
    if CurHeight > FHeight then
      FHeight := CurHeight + 2;
    R := Rect(FStarts[Column], FCurY,
              FStarts[Column] + FWidths[Column], FCurY + CurHeight);
    if (R.Left > FMaxX) or (R.Right - R.Left < TwoEms) then
      Exit;
    if R.Right > FMaxX then
      R.Right := FMaxX;
    DrawText(Handle, PChar(S), -1, R,
      AlignFlags[FListView.Columns[Column].Alignment] or DT_TOP or DT_SINGLELINE);
  end;
end;

end.
